@@ -70,7 +70,7 @@ import java.util.Set;
 import java.util.StringTokenizer;
 import java.util.logging.Logger;
 
-import static java.util.logging.Level.WARNING;
+import static java.util.logging.Level.*;
 import static javax.servlet.http.HttpServletResponse.*;
 
 /**
@@ -107,6 +107,13 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
      * @see #parseMultipartFormData()
      */
     private Map<String, FileItem> parsedFormData;
+    
+    /**
+     * If the request is "multipart/form-data", the form field only parts of the parsed result go here.
+     *
+     * @see #parseMultipartFormData()
+     */    
+    private Map<String, String> parsedFormDataFormFields;
 
     private BindInterceptor bindInterceptor = BindInterceptor.NOOP;
 
@@ -152,6 +159,71 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
         return stapler.getServletContext();
     }
 
+    @Override
+    public String getParameter(String name) {
+        if(isMultipart()) {
+            Map<String, String> data = getFormDataFormFields();
+            String value = data.get(name);
+            if (value != null) {
+                return value;
+            }
+            // Fallback ...
+        }
+        return super.getParameter(name);
+    }
+
+    @Override
+    public Map getParameterMap() {
+        Map parameterMap = super.getParameterMap();
+        if(isMultipart()) {
+            Map<String, String> data = getFormDataFormFields();
+            parameterMap.putAll(data);
+        }
+        return parameterMap;
+    }
+
+    @Override
+    public Enumeration getParameterNames() {
+        if(!isMultipart()) {
+            return super.getParameterNames();
+        }
+        Map<String, String> data = getFormDataFormFields();
+        if (data.isEmpty()) {
+            return super.getParameterNames();
+        }
+        
+        List<String> paramNames = Collections.<String>list(super.getParameterNames());
+        paramNames.addAll(data.keySet());
+        
+        return Collections.enumeration(paramNames);
+    }
+
+    @Override
+    public String[] getParameterValues(String name) {
+        if(!isMultipart()) {
+            return super.getParameterValues(name);
+        }
+        Map<String, String> data = getFormDataFormFields();
+        if (data.isEmpty()) {
+            return super.getParameterValues(name);
+        }
+        
+        String formFieldVal = data.get(name);
+        if (formFieldVal == null) {
+            return super.getParameterValues(name);
+        }
+
+        String[] values = super.getParameterValues(name);
+        if (values == null) {
+            values = new String[0];
+        }
+        String[] extValues = new String[values.length + 1];
+        System.arraycopy(values, 0, extValues, 0, values.length);
+        extValues[extValues.length - 1] = formFieldVal;
+
+        return extValues;
+    }
+
     public String getRequestURIWithQueryString() {
         String s = getRequestURI();
         String q = getQueryString();
@@ -292,10 +364,14 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
     }
 
     public BindInterceptor setBindListener(BindInterceptor bindListener) {
-        return setBindInterceptpr(bindListener);
+        return setBindInterceptor(bindListener);
     }
 
     public BindInterceptor setBindInterceptpr(BindInterceptor bindListener) {
+        return setBindInterceptor(bindListener);
+    }
+
+    public BindInterceptor setBindInterceptor(BindInterceptor bindListener) {
         BindInterceptor o = this.bindInterceptor;
         this.bindInterceptor = bindListener;
         return o;
@@ -568,12 +644,21 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
                 if(l==null) {// single value conversion
                     try {
                         Class actualType = type;
+                        String className = null;
                         if(j.has("stapler-class")) {
+                            // deprecated as of 2.4-jenkins-4 but left here for a while until we are sure nobody uses this
+                            className = j.getString("stapler-class");
+                            LOGGER.log(FINE, "stapler-class is deprecated in favor of $class: {0}", className);
+                        }
+                        if(j.has("$class")) {
+                            className = j.getString("$class");
+                        }
+
+                        if (className != null) {
                             // sub-type is specified in JSON.
                             // note that this can come from malicious clients, so we need to make sure we don't have security issues.
 
                             ClassLoader cl = stapler.getWebApp().getClassLoader();
-                            String className = j.getString("stapler-class");
                             try {
                                 Class<?> subType = cl.loadClass(className);
                                 if(!actualType.isAssignableFrom(subType))
@@ -647,7 +732,7 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
             } else {// single value in a collection
                 Converter converter = Stapler.lookupConverter(l.itemType);
                 if (converter==null)
-                    throw new IllegalArgumentException("Unable to convert to "+type);
+                    throw new IllegalArgumentException("Unable to convert to "+l.itemType);
 
                 l.add(converter.convert(type,o));
                 return l.toCollection();
@@ -687,10 +772,26 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
             }
         }
 
-        return injectSetters(invokeConstructor(c, args), j, Arrays.asList(names));
+        Object o = injectSetters(invokeConstructor(c, args), j, Arrays.asList(names));
+        o = bindResolve(o,j);
+
+        return o;
     }
 
     /**
+     * Calls {@link DataBoundResolvable#bindResolve(StaplerRequest, JSONObject)} if the object has it.
+     */
+    private Object bindResolve(Object o, JSONObject src) {
+        if (o instanceof DataBoundResolvable) {
+            DataBoundResolvable dbr = (DataBoundResolvable) o;
+            o = dbr.bindResolve(this,src);
+        }
+        return o;
+    }
+
+    /**
+     * Performs {@link DataBoundSetter} injections.
+     *
      * @param exclusions
      *      Properties that are already injected through the constructor, thus not subject of the setter injection.
      */
@@ -833,22 +934,38 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
         if(parsedFormData!=null)    return;
 
         parsedFormData = new HashMap<String,FileItem>();
+        parsedFormDataFormFields = new HashMap<String, String>();
         ServletFileUpload upload = new ServletFileUpload(new DiskFileItemFactory());
         try {
-            for( FileItem fi : (List<FileItem>)upload.parseRequest(this) )
+            for( FileItem fi : (List<FileItem>)upload.parseRequest(this) ) {
                 parsedFormData.put(fi.getFieldName(),fi);
+                if (fi.isFormField()) {
+                    parsedFormDataFormFields.put(fi.getFieldName(),fi.getString());
+                }
+            }
         } catch (FileUploadException e) {
             throw new ServletException(e);
         }
     }
 
+    /**
+     * A version of parseMultipartFormData() that doesn't throw exceptions.
+     */
+    private Map<String, String> getFormDataFormFields() {
+        try {
+            parseMultipartFormData();
+        } catch (Exception e) {
+            LOGGER.log(SEVERE, "Error parsing multipart/form-data.", e);
+        }
+        return parsedFormDataFormFields;
+    }
+
     public JSONObject getSubmittedForm() throws ServletException {
         if(structuredForm==null) {
-            String ct = getContentType();
             String p = null;
             boolean isSubmission; // for error diagnosis, if something is submitted, set to true
 
-            if(ct!=null && ct.startsWith("multipart/")) {
+            if(isMultipart()) {
                 isSubmission=true;
                 parseMultipartFormData();
                 FileItem item = parsedFormData.get("json");
@@ -892,6 +1009,11 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
         return structuredForm;
     }
 
+    private boolean isMultipart() {
+        String ct = getContentType();
+        return (ct!=null && ct.startsWith("multipart/"));
+    }
+
     public FileItem getFileItem(String name) throws ServletException, IOException {
         parseMultipartFormData();
         if(parsedFormData==null)    return null;
