@@ -333,7 +333,7 @@ public interface StaplerRequest extends HttpServletRequest {
      * Instantiates a new object by injecting constructor parameters from the form parameters.
      *
      * <p>
-     * The given class must have a constructor annotated with '@stapler-constructor',
+     * The given class must have a constructor annotated with '&#64;stapler-constructor',
      * and must be processed by the maven-stapler-plugin, so that the parameter names
      * of the constructor is available at runtime.
      *
@@ -395,7 +395,7 @@ public interface StaplerRequest extends HttpServletRequest {
      * <h3>Sub-typing</h3>
      * <p>
      * In the above example, a new instance of <tt>Bar</tt> was created,
-     * but you can also create a subtype of Bar by having the 'stapler-class' property in
+     * but you can also create a subtype of Bar by having the '$class' property in
      * JSON like this:
      *
      * <pre>
@@ -404,7 +404,7 @@ public interface StaplerRequest extends HttpServletRequest {
      *   public BarEx(int a, int b, int c) {}
      * }
      *
-     * { y:"text", z:true, bar: { stapler-class:"p.k.g.BarEx", a:1, b:2, c:3 } }
+     * { y:"text", z:true, bar: { $class:"p.k.g.BarEx", a:1, b:2, c:3 } }
      * </pre>
      *
      * <p>
@@ -463,12 +463,18 @@ public interface StaplerRequest extends HttpServletRequest {
 
     /**
      * @deprecated
-     * Typo. Use {@link #setBindInterceptpr(BindInterceptor)}
+     * Typo. Use {@link #setBindInterceptor(BindInterceptor)}
      */
     BindInterceptor setBindListener(BindInterceptor bindListener);
 
+    /**
+     * @deprecated
+     * Typo. Use {@link #setBindInterceptor(BindInterceptor)}
+     */
     BindInterceptor setBindInterceptpr(BindInterceptor bindListener);
 
+    BindInterceptor setBindInterceptor(BindInterceptor bindListener);
+
     /**
      * Gets the content of the structured form submission.
      *
