@@ -23,8 +23,6 @@
 
 package org.kohsuke.stapler.export;
 
-import org.kohsuke.stapler.export.TreePruner.ByDepth;
-
 import java.io.IOException;
 import java.io.InputStream;
 import java.lang.reflect.Field;
@@ -36,9 +34,15 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Properties;
 import java.util.Set;
+import java.util.logging.Level;
+import java.util.logging.Logger;
+
 import javax.annotation.CheckForNull;
 import javax.annotation.Nullable;
 
+import com.google.common.base.Predicate;
+import org.kohsuke.stapler.export.TreePruner.ByDepth;
+
 /**
  * Writes all the property of one {@link ExportedBean} to {@link DataWriter}.
  *
@@ -61,10 +65,12 @@ public class Model<T> {
     /*package*/ final int defaultVisibility;
 
     /**
-     * Lazily loaded "*.javadoc" file for this model. 
+     * Lazily loaded "*.javadoc" file for this model.
      */
     private volatile Properties javadoc;
 
+    private final Set<String> propertyNames = new HashSet<String>();
+
     /*package*/ Model(ModelBuilder parent, Class<T> type, @CheckForNull Class<?> propertyOwner, @Nullable String property) throws NotExportableException {
         this.parent = parent;
         this.type = type;
@@ -73,7 +79,7 @@ public class Model<T> {
             throw propertyOwner != null ? new NotExportableException(type, propertyOwner, property) : new NotExportableException(type);
         }
         this.defaultVisibility = eb.defaultVisibility();
-        
+
         Class<? super T> sc = type.getSuperclass();
         if(sc!=null && sc.getAnnotation(ExportedBean.class)!=null)
             superModel = parent.get(sc);
@@ -92,13 +98,21 @@ public class Model<T> {
 
         for( Method m : type.getMethods() ) {
             if(m.getDeclaringClass()!=type) continue;
+            if(m.isSynthetic() && m.isBridge()) continue;
             Exported exported = m.getAnnotation(Exported.class);
-            if(exported !=null)
-                properties.add(new MethodProperty(this,m, exported));
+            if(exported !=null) {
+                if (m.getParameterTypes().length > 0) {
+                    LOGGER.log(Level.WARNING, "Method " + m.getName() + " of " + type.getName() + " is annotated @Exported but requires arguments");
+                } else {
+                    properties.add(new MethodProperty(this,m, exported));
+                }
+            }
         }
 
         this.properties = properties.toArray(new Property[properties.size()]);
         Arrays.sort(this.properties);
+        for (Property p : properties)
+            this.propertyNames.add(p.name);
 
         parent.models.put(type,this);
     }
@@ -110,6 +124,28 @@ public class Model<T> {
         return Collections.unmodifiableList(Arrays.asList(properties));
     }
 
+    /**
+     * Does a property exist strictly in this class?
+     */
+    /*package*/ final Predicate<String> HAS_PROPERTY_NAME = new Predicate<String>() {
+        @Override
+        public boolean apply(@Nullable String name) {
+            return propertyNames.contains(name);
+        }
+    };
+    /**
+     * Does a property exist strictly in this class or its ancestors?
+     */
+    /*package*/ final Predicate<String> HAS_PROPERTY_NAME_IN_ANCESTORY = new Predicate<String>() {
+        @Override
+        public boolean apply(@Nullable String name) {
+            for (Model m=Model.this; m!=null; m=m.superModel)
+                if (m.propertyNames.contains(name))
+                    return true;
+            return false;
+        }
+    };
+
     /**
      * Loads the javadoc list and returns it as {@link Properties}.
      *
@@ -143,7 +179,7 @@ public class Model<T> {
      * Writes the property values of the given object to the writer.
      */
     public void writeTo(T object, DataWriter writer) throws IOException {
-        writeTo(object,0,writer);
+        writeTo(object, 0, writer);
     }
 
     /**
@@ -153,8 +189,13 @@ public class Model<T> {
      *      Controls which portion of the object graph will be sent to the writer.
      */
     public void writeTo(T object, TreePruner pruner, DataWriter writer) throws IOException {
+        try {
+            writer.type(null,object.getClass());
+        } catch (AbstractMethodError _) {
+            // legacy client that doesn't understand this
+        }
         writer.startObject();
-        writeNestedObjectTo(object, pruner, writer, Collections.<String>emptySet());
+        writeNestedObjectTo(object, pruner, writer);
         writer.endObject();
     }
 
@@ -165,30 +206,27 @@ public class Model<T> {
      *      This parameters controls how much data we'd be writing,
      *      by adding bias to the sub tree cutting.
      *      A property with {@link Exported#visibility() visibility} X will be written
-     *      if the current depth Y and baseVisibility Z satisfies X+Z>Y.
+     *      if the current depth Y and baseVisibility Z satisfies {@code X + Z > Y}.
      *
      *      0 is the normal value. Positive value means writing bigger tree,
      *      and negative value means writing smaller trees.
      *
      * @deprecated as of 1.139
      */
+    @Deprecated
     public void writeTo(T object, int baseVisibility, DataWriter writer) throws IOException {
         writeTo(object,new ByDepth(1-baseVisibility),writer);
     }
 
-    void writeNestedObjectTo(T object, TreePruner pruner, DataWriter writer, Set<? extends String> blacklist) throws IOException {
+    void writeNestedObjectTo(T object, TreePruner pruner, DataWriter writer) throws IOException {
         if (superModel != null) {
-            Set<String> superBlacklist = new HashSet<String>(blacklist);
-            for (Property p : properties) {
-                superBlacklist.add(p.name);
-            }
-            superModel.writeNestedObjectTo(object, pruner, writer, superBlacklist);
+            superModel.writeNestedObjectTo(object, new FilteringTreePruner(HAS_PROPERTY_NAME,pruner), writer);
         }
 
         for (Property p : properties) {
-            if (!blacklist.contains(p.name)) {
-                p.writeTo(object,pruner,writer);
-            }
+            p.writeTo(object, pruner, writer);
         }
     }
+
+    private static final Logger LOGGER = Logger.getLogger(Model.class.getName());
 }
