@@ -23,23 +23,20 @@
 
 package org.kohsuke.stapler.export;
 
+import org.jvnet.tiger_types.Types;
 import org.kohsuke.stapler.export.TreePruner.ByDepth;
 
 import java.io.IOException;
+import java.lang.reflect.Array;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Type;
-import java.lang.reflect.Array;
 import java.net.URL;
 import java.util.Arrays;
 import java.util.Calendar;
-import java.util.Collection;
-import java.util.Collections;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
-import java.util.logging.Level;
-import java.util.logging.Logger;
 
 /**
  * Exposes one {@link Exported exposed property} of {@link ExportedBean} to
@@ -71,17 +68,32 @@ public abstract class Property implements Comparable<Property> {
      */
     public final boolean inline;
 
+    /**
+     * @see Exported#merge()
+     */
+    public final boolean merge;
+
+    /**
+     * @see Exported#skipNull()
+     */
+    private final boolean skipNull;
+
     private String[] verboseMap;
 
-    Property(Model parent, String name, Exported exported) {
+    private final Type type;
+
+    Property(Model parent, String name, Type type, Exported exported) {
         this.parent = parent;
         this.owner = parent.parent;
         this.name = exported.name().length()>1 ? exported.name() : name;
+        this.type = type;
         int v = exported.visibility();
         if(v==0)
             v = parent.defaultVisibility;
         this.visibility = v;
         this.inline = exported.inline();
+        this.merge = exported.merge();
+        this.skipNull = exported.skipNull();
         String[] s = exported.verboseMap().split("/");
         if (s.length<2)
             this.verboseMap = null;
@@ -111,9 +123,26 @@ public abstract class Property implements Comparable<Property> {
         TreePruner child = pruner.accept(object, this);
         if (child==null)        return;
 
-        try {
+        Object d = safeGetValue(object);
+
+        if (d==null && skipNull) { // don't write anything
+            return;
+        }
+        if (merge) {
+            // merged property will get all its properties written here
+            if (d != null) {
+                Model model = owner.get(d.getClass(), parent.type, name);
+                model.writeNestedObjectTo(d, new FilteringTreePruner(parent.HAS_PROPERTY_NAME_IN_ANCESTORY,child), writer);
+            }
+        } else {
             writer.name(name);
-            writeValue(getValue(object),child,writer);
+            writeValue(type, d, child, writer);
+        }
+    }
+
+    private Object safeGetValue(Object o) throws IOException {
+        try {
+            return getValue(o);
         } catch (IllegalAccessException e) {
             IOException x = new IOException("Failed to write " + name);
             x.initCause(e);
@@ -135,113 +164,131 @@ public abstract class Property implements Comparable<Property> {
     /**
      * Writes one value of the property to {@link DataWriter}.
      */
-    private void writeValue(Object value, TreePruner pruner, DataWriter writer) throws IOException {
-        writeValue(value,pruner,writer,false);
+    private void writeValue(Type expected, Object value, TreePruner pruner, DataWriter writer) throws IOException {
+        writeValue(expected,value,pruner,writer,false);
     }
 
     /**
      * Writes one value of the property to {@link DataWriter}.
      */
-    private void writeValue(Object value, TreePruner pruner, DataWriter writer, boolean skipIfFail) throws IOException {
+    private void writeValue(Type expected, Object value, TreePruner pruner, DataWriter writer, boolean skipIfFail) throws IOException {
         if(value==null) {
             writer.valueNull();
             return;
         }
 
         if(value instanceof CustomExportedBean) {
-            writeValue(((CustomExportedBean)value).toExportedObject(),pruner,writer);
+            writeValue(expected,((CustomExportedBean)value).toExportedObject(),pruner,writer);
             return;
         }
 
         Class c = value.getClass();
 
-        if(STRING_TYPES.contains(c)) {
-            writer.value(value.toString());
-            return;
-        }
-        if(PRIMITIVE_TYPES.contains(c)) {
-            writer.valuePrimitive(value);
-            return;
-        }
-        if(c.getComponentType()!=null) { // array
-            Range r = pruner.getRange();
-            writer.startArray();
-            if (value instanceof Object[]) {
-                // typical case
-                for (Object item : r.apply((Object[]) value)) {
-                    writeValue(item,pruner,writer,true);
-                }
-            } else {
-                // more generic case
-                int len = Math.min(r.max,Array.getLength(value));
-                for (int i=r.min; i<len; i++) {
-                    writeValue(Array.get(value,i),pruner,writer,true);
-                }
-            }
-            writer.endArray();
-            return;
-        }
-        if(value instanceof Collection) {
-            writer.startArray();
-            for (Object item : pruner.getRange().apply((Collection) value)) {
-                writeValue(item,pruner,writer,true);
-            }
-            writer.endArray();
-            return;
-        }
-        if(value instanceof Map) {
-            if (verboseMap!=null) {// verbose form
-                writer.startArray();
-                for (Map.Entry e : ((Map<?,?>) value).entrySet()) {
-                    writer.startObject();
-                    writer.name(verboseMap[0]);
-                    writeValue(e.getKey(),pruner,writer);
-                    writer.name(verboseMap[1]);
-                    writeValue(e.getValue(),pruner,writer);
-                    writer.endObject();
-                }
-                writer.endArray();
-            } else {// compact form
-                writer.startObject();
-                for (Map.Entry e : ((Map<?,?>) value).entrySet()) {
-                    writer.name(e.getKey().toString());
-                    writeValue(e.getValue(),pruner,writer);
-                }
-                writer.endObject();
-            }
-            return;
-        }
-        if(value instanceof Date) {
-            writer.valuePrimitive(((Date) value).getTime());
-            return;
-        }
-        if(value instanceof Calendar) {
-            writer.valuePrimitive(((Calendar) value).getTimeInMillis());
-            return;
-        }
-        if(value instanceof Enum) {
-            writer.value(value.toString());
-            return;
-        }
-
-        // otherwise handle it as a bean
-        writer.startObject();
-        Model model = null;
+        Model model;
         try {
             model = owner.get(c, parent.type, name);
-        } catch (NotExportableException e) {
-            if (skipIfFail) {
-                Logger.getLogger(Property.class.getName()).log(Level.FINE, e.getMessage());
-            } else {
-                throw e;
+        } catch (NotExportableException ex) {
+            if(STRING_TYPES.contains(c)) {
+                writer.value(value.toString());
+                return;
             }
-            // otherwise ignore this error by writing empty object
+            if(PRIMITIVE_TYPES.contains(c)) {
+                writer.valuePrimitive(value);
+                return;
+            }
+            Class act = c.getComponentType();
+            if (act !=null) { // array
+                Range r = pruner.getRange();
+                writer.startArray();
+                if (value instanceof Object[]) {
+                    // typical case
+                    for (Object item : r.apply((Object[]) value)) {
+                        writeValue(act,item,pruner,writer,true);
+                    }
+                } else {
+                    // more generic case
+                    int len = Math.min(r.max, Array.getLength(value));
+                    for (int i=r.min; i<len; i++) {
+                        writeValue(act,Array.get(value,i),pruner,writer,true);
+                    }
+                }
+                writer.endArray();
+                return;
+            }
+            if(value instanceof Iterable) {
+                writer.startArray();
+                Type expectedItemType = Types.getTypeArgument(expected, 0, null);
+                for (Object item : pruner.getRange().apply((Iterable) value)) {
+                    writeValue(expectedItemType,item,pruner,writer,true);
+                }
+                writer.endArray();
+                return;
+            }
+            if(value instanceof Map) {
+                if (verboseMap!=null) {// verbose form
+                    writer.startArray();
+                    for (Map.Entry e : ((Map<?,?>) value).entrySet()) {
+                        writeStartObjectNullType(writer);
+                        writer.name(verboseMap[0]);
+                        writeValue(null,e.getKey(),pruner,writer);
+                        writer.name(verboseMap[1]);
+                        writeValue(null,e.getValue(),pruner,writer);
+                        writer.endObject();
+                    }
+                    writer.endArray();
+                } else {// compact form
+                    writeStartObjectNullType(writer);
+                    for (Map.Entry e : ((Map<?,?>) value).entrySet()) {
+                        writer.name(e.getKey().toString());
+                        writeValue(null,e.getValue(),pruner,writer);
+                    }
+                    writer.endObject();
+                }
+                return;
+            }
+            if(value instanceof Date) {
+                writer.valuePrimitive(((Date) value).getTime());
+                return;
+            }
+            if(value instanceof Calendar) {
+                writer.valuePrimitive(((Calendar) value).getTimeInMillis());
+                return;
+            }
+            if(value instanceof Enum) {
+                writer.value(value.toString());
+                return;
+            }
+
+            if (skipIfFail) {
+                writer.startObject();
+                writer.endObject();
+                return;
+            }
+
+            throw ex;
         }
-        if(model!=null)
-            model.writeNestedObjectTo(value, pruner, writer, Collections.<String>emptySet());
+
+        try {
+            writer.type(expected, value.getClass());
+        } catch (AbstractMethodError _) {
+            // legacy impl that doesn't understand it
+        }
+
+
+        writer.startObject();
+        model.writeNestedObjectTo(value, pruner, writer);
         writer.endObject();
     }
 
+    private void writeStartObjectNullType(DataWriter writer) throws IOException {
+        try {
+            writer.type(null,null);
+        } catch (AbstractMethodError _) {
+            // legacy client that doesn't understand this
+        }
+        writer.startObject();
+    }
+
     /**
      * Gets the value of this property from the bean.
      */
