@@ -1,8 +1,10 @@
 package org.kohsuke.stapler.jsr269;
 
 import java.util.Collections;
+
 import net.java.dev.hickory.testing.Compilation;
 import org.junit.Test;
+
 import static org.junit.Assert.*;
 
 public class QueryParameterAnnotationProcessorTest {
@@ -17,7 +19,7 @@ public class QueryParameterAnnotationProcessorTest {
                 addLine("  public void doAnother(@QueryParameter(\"ignoredHere\") String name, @QueryParameter String address) {}").
                 addLine("}");
         compilation.doCompile(null, "-source", "6");
-        assertEquals(Collections.emptyList(), Utils.filterSupportedSourceVersionWarnings(compilation.getDiagnostics()));
+        assertEquals(Collections.emptyList(), compilation.getDiagnostics());
         assertEquals("key", Utils.getGeneratedResource(compilation, "some/pkg/Stuff/doOneThing.stapler"));
         assertEquals("name,address", Utils.getGeneratedResource(compilation, "some/pkg/Stuff/doAnother.stapler"));
     }
